<?php

// PageIndex.class.php
//
// $Id: PageIndex.class.php 1581 2009-04-09 11:12:06Z tim $
//
// (c)2003-2004 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// The default max number of pages which will be shown in a pageindex
define('PAGEINDEX_LIMIT_PAGES_DEFAULT',15);
// The default pages to skip if number of pages exceeds PAGEINDEX_LIMIT_PAGES_DEFAULT
define('PAGEINDEX_SKIP_PAGES_DEFAULT',10);

/**
 * This class allows the generation of nice HTML "page indexes" for when you
 * have a data resultset containing too many rows to show on a page. The key
 * benefit over most other implementations is that it handles very large 
 * datasets elegantly and intelligently; if you have for example 800 pages, it
 * will not show 1,000 links; it will show something like this (if you are on 
 * page 256):
 * 
 * 1...100...200...254..255..256..257..258...300..400...500...600...700...800
 *
 * It's fully configurable.
 *
 * To use it, you just do something like this (using the above example):
 *
 * // create an object. the params are:
 * // number of pages
 * // current page number
 * // the URL that links should point to. "{}" will be replaced with the chosen page ID 
 * // assume $page is a variable that contains the current page of results that we are looking at
 * $PageIndexer = new PageIndex(800,$page,'foo.php?page={}');
 * print $PageIndexer->generate();
 *
 * Check the class properties for configuration options.
 */
class PageIndex {
	
	var $Pages = 0;
	var $CurrentPage;
	var $URL;
	
	/**
	 * what string to separate the output links with
	 */
	var $Separator=' ';
	
	/**
	 * how many links to show, at a maximum
	 */
	var $LimitPages = PAGEINDEX_LIMIT_PAGES_DEFAULT;
	
	/**
	 * how many pages to "skip" if LimitPages is exceeded
	 */
	var $SkipPages = PAGEINDEX_SKIP_PAGES_DEFAULT;
	
	/**
	 * whether to show "Next" and "Previous" links
	 */
	var $NextPrev=false;
	
	/**
	 * what text to show for a "Previous" link
	 */
	var $Prev='&lt;&lt;';
	
	/**
	 * what text to show for a "Next" link
	 */
	var $Next='&gt;&gt;';
	
	/**
	 * what to print out at the start of the page index
	 */
	var $Header = "<div class='pageindex'>";
	
	/**
	 * text to print out at the end of the page index
	 */
	var $Footer = "</div>";
	
	/** 
	 * whether to "hide" the Next/Prev link (if NextPrev enabled) when the user 
	 * is viewing the first/last page. If off, then "Next" or "Prev" will still 
	 * be shown, but won't be a link. E.g. if the user is on page 1, if 
	 * HideEndNextPrev is on then there will only be a "Next" link not a "Prev" 
	 * link.
	 */
	var $HideEndNextPrev = false;
	
	/**
	 * whether to make the current page an actual link or not, or just show the number as plain text
	 */
	var $LinkCurrentPage = false;
	var $_Output = '';
	
	function PageIndex($Pages,$CurrentPage,$URL)
	{
		$this->Pages = $Pages;
		$this->CurrentPage = $CurrentPage;
		$this->URL = $URL;
		return true;
	}
	
	
	function generate() {
		if ($this->Pages > 1) {
			$this->_Output = $this->Header;
			
			// Print 'Prev'
			if ($this->NextPrev) {
				if ($this->CurrentPage != 1) {
					$this->_Output .= "<a href='".str_replace('{}',$this->CurrentPage-1,$this->URL)."'>$this->Prev</a>";
				} else {
					if (!$this->HideEndNextPrev) {
						$this->_Output .= $this->Prev;
					}
				}
			}
			
			// Print page links
			if ($this->Pages <= $this->LimitPages) {
				for ($Page = 1; $Page <= $this->Pages; $Page++) {
					$this->_showPage($Page);
					$this->_Output .= $this->Separator;
				}
			} else {
				$Count = $this->SkipPages;	// Show a link every few pages
				$PrevPageShown = 0;
				for ($Page = 1; $Page <= $this->Pages; $Page++) {
					// Check if we are within SkipPages/2 pages of the actual page
					if (abs($this->CurrentPage - $Page) > ($this->SkipPages/2)) {
						if (($Count++ == $this->SkipPages) || $Page == $this->Pages) {
							$this->_showSeparator($Page,$PrevPageShown);
							$this->_showPage($Page);
							$Count = 0;
							$PrevPageShown = $Page;
						}
					} else {
						$this->_showSeparator($Page,$PrevPageShown);
						$this->_showPage($Page);
						$PrevPageShown = $Page;
					}
				}
			}
			
			// Print 'Next'
			if ($this->NextPrev) {
				if ($this->CurrentPage != $this->Pages) {
					$this->_Output .= "<a href='".str_replace('{}',$this->CurrentPage+1,$this->URL)."'>$this->Next</a>";
				} else {
					if (!$this->HideEndNextPrev) {
						$this->_Output .= $this->Next;
					}
				}
			}
			$this->_Output .= $this->Footer;
			return $this->_Output;
		}
	}
	
	function _showPage($Page)
	{
		if ($Page != $this->CurrentPage) {
			$this->_Output .= "<a href='".str_replace('{}',$Page,$this->URL)."'>$Page</a>";
		} else {
			if ($this->LinkCurrentPage) {
				$this->_Output .= "<a href='".str_replace('{}',$Page,$this->URL)."' class='pageindex-current'>$Page</a>";
			} else {
				$this->_Output .= "<span class='pageindex-current'>$Page</span>";
			}
		}
	}
	
	function _showSeparator($Page,$PrevPageShown)
	{
		if ($Page > $PrevPageShown+1) {
			$this->_Output .= '..';
		} else {
			if ($Page > 1) $this->_Output .= $this->Separator;
		}
	}
}
