<?php

// XVar.class.php - functions to assist when using variables
//									including anti-XSS measures
//
// $Id: XVar.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2002-2005 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

define('XVAR_INT',1);
define('XVAR_ALPHA',2);
define('XVAR_ALNUM',3);
define('XVAR_STANDARD',4);
define('XVAR_FLOAT',5);
define('XVAR_LIBERAL',6);
define('XVAR_BOOL',7);
define('XVAR_DB_BOOL',8);
define('XVAR_ALPHA_EXTENDED',9);
define('XVAR_ALPHA_EXTENDED_ADDITIONAL',10);
define('XVAR_ALNUM_EXTENDED',11);
define('XVAR_ALNUM_EXTENDED_ADDITIONAL',12);
define('XVAR_SAFE_REFERENCE',13);
define('XVAR_ANY',14);
define('XVAR_NO_LINEBREAKS',15);

class XVar {

	/**
	 * Register a variable from user input (a request parameter) in the global scope. 
	 *
	 * This function provides a safer alternative to register_globals.
	 * 
	 * @param string $orig  Name of variable to register
	 * @param string $type  Type to register variable as (one of XVAR_* constants)
	 * @param bool $set_array_element_if_unset  Whether to set array elements if they don't exist in the input data
	 * @param bool $check_only  If set, only test to see if the input ($orig) is valid according to the specified type ($type)
	 * @return mixed  The cleaned input
	 */
	function register ($orig, $type=XVAR_ANY, $set_array_element_if_unset = true, $check_only = false) {
		$array = false;
		if (ereg('(.*)\[(.*)\]',$orig, $matches)) {	// Look for arrays
			$array = true;
			$orig = $matches[1];		// Base variable name
			$index = $matches[2];	// Array index
		}
		global $$orig;
		
		$output = false;
		if (isset($_REQUEST[$orig])) {
			if (!$array) {
				$variable = $_REQUEST[$orig];
			} else {
				$arraybase = $_REQUEST[$orig];
				if (isset($arraybase[$index])) {
					$variable = $arraybase[$index];
				} elseif ($set_array_element_if_unset) {
					// Set the variable IRRELEVANT of whether it is set in $_REQUEST.
					// This is often required because arrays are mainly used for passing object params, 
					// and you want ALL params set if you are going to use DBRow::fillFromArray().
					$variable = '';
				}
			}

		
			$settype = '';
			$regex = '';
			$extrachars = "";	// Common high-bit alphabetic characters

			switch($type) {
				case XVAR_INT:
					$settype = 'integer';
					break;
				case XVAR_FLOAT:
					$settype = 'float';
					break;
				case XVAR_ALPHA:
					$regex = "[^[:alpha:]]";
					$settype = 'string';
					break;
				case XVAR_ALPHA_EXTENDED:
					$regex = "[^[:alpha:]$extrachars]";
					$settype = 'string';
					break;
				case XVAR_ALPHA_EXTENDED_ADDITIONAL:
					$regex = "[^[:alpha:] $extrachars\-_]";
					$settype = 'string';
					break;
				case XVAR_ALNUM:
					$regex = "[^[:alnum:]]";
					$settype = 'string';
					break;
				case XVAR_ALNUM_EXTENDED:
					$regex = "[^[:alnum:]$extrachars]";
					$settype = 'string';
					break;
				case XVAR_ALNUM_EXTENDED_ADDITIONAL:
					$regex = "[^[:alnum:] $extrachars\-_]";
					$settype = 'string';
					break;
				case XVAR_SAFE_REFERENCE:
					$regex = "[^[:alnum:]_\-]";
					$settype = 'string';
					break;
				case XVAR_LIBERAL:
					$regex = "[^[:alnum:] $extrachars&\@#!;:=\?\^\(\)~\+\$%_/\"\'\-\.\,]";
					$settype = 'string';
					break;
				case XVAR_BOOL:
					$regex = "[^01]*";
					$settype = 'integer';
					break;
				case XVAR_DB_BOOL:	// FIXME: Needs to check for DB_BOOLEAN_[TRUE|FALSE]
					$regex = "[^01]*";
					break;
				case XVAR_STANDARD:
					$output = strip_tags($variable);
					break;
				case XVAR_ANY:
					$output = $variable;
					break;
				// useful for data to be used in email headers
				case XVAR_NO_LINEBREAKS:
					$output = str_replace("\n",'\n',$variable);
					$output = str_replace("\r",'\n',$output);
					break;
			}
			if (empty($output) && isset($variable)) {
				$output = stripslashes($variable);
				if (!empty($settype)) {
					settype ($output, $settype);
				}
				if (!empty($regex)) {
					$output = eregi_replace($regex,'',$output);
				}
				$output = addslashes($output);
			}
			if (isset($variable) && !$check_only) {
				if ($array) {
					${$orig}[$index] = $output;
				} else {
					$$orig = $output;
				}
			}
		}
		return $output;	// backwards compat
	}
	
	/**
	 * Check if the given variable conforms to the given type. Returns TRUE or FALSE.
	 * 
	 * @param string $Variable  The name of the variable
	 * @param string $Type  An XVAR_* type
	 * @return bool
	*/
	function isValid ($Variable, $Type=XVAR_ANY)
	{
		$Output = XVar::register($Variable, $Type, true, true);
		
		$array = false;
		$Original = null;
		if (ereg('(.*)\[(.*)\]',$Variable, $matches)) {	// Look for arrays
			$array = true;
			$Variable = $matches[1]; // Base variable name
			$index = $matches[2]; // Array index
		}
		if (isset($_REQUEST[$Variable])) {
			if (!$array) {
				$Original = $_REQUEST[$Variable];
			} else {
				$arraybase = $_REQUEST[$Variable];
				if (isset($arraybase[$index])) {
					$Original = $arraybase[$index];
				}
			}
		}
		
		if ($Original == $Output) {
			return true;
		}
		return false;
	}
	
	function getTypeDescription($Type)
	{
		switch ($Type) {
		case XVAR_INT:
			return 'a round (integer) number';
			break;
		case XVAR_FLOAT:
			return 'a number';
			break;
		case XVAR_ALPHA:
			return 'only letters of the alphabet';
			break;
		case XVAR_ALPHA_EXTENDED:
			return 'only letters of the alphabet';
			break;
		case XVAR_ALPHA_EXTENDED_ADDITIONAL:
			return 'only letters of the alphabet, hyphens and underscores';
			break;
		case XVAR_ALNUM:
			return 'only letters of the alphabet and numbers';
			break;
		case XVAR_ALNUM_EXTENDED:
			return 'only letters of the alphabet and numbers';
			break;
		case XVAR_ALNUM_EXTENDED_ADDITIONAL:
			return 'only letters of the alphabet, numbers, hyphens and underscores';
			break;
		case XVAR_SAFE_REFERENCE:
			return 'only letters of the alphabet, numbers, hyphens and underscores';
			break;
		case XVAR_LIBERAL:
			return 'only letters of the alphabet, numbers, and the following symbols: ;&\@#!;:=\?\^\(\)~\+\$%_/\"\'\-\.\,]';
			break;
		case XVAR_BOOL:
			return 'only the numbers 0 or 1';
			break;
		case XVAR_DB_BOOL:	// FIXME: Needs to check for DB_BOOLEAN_[TRUE|FALSE]
			return 'only the numbers 0 or 1';
			break;
		case XVAR_STANDARD:
			return 'any text except HTML tags';
			break;
		case XVAR_ANY:
			return 'any text';
			break;
		}
	}
	
	function registerValidated ($Variable, $Type, &$Errors)
	{
		XVar::register($Variable, XVAR_ANY);
		if (!XVar::isValid($Variable, $Type)) {
			$Desc = XVar::getTypeDescription($Type);
			$Errors[] = "The value for '".ucfirst(str_replace('_',' ',preg_replace("/^[a-z0-9]\[(.+)\]$/i",'\1',$Variable)))."' is not valid. It may contain $Desc.";
			return false;
		}
		return true;
	}
}
